-- this is basically the example from fig.7.6 of the following textbook:
-- P.P. Chu, FPGA Prototyping by VHDL Examples: Xilinx Spartan-3 Version, Wiley 2008.

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;


entity uart_test_impl1 is
    port(
        clk: in std_logic; -- 50 MHz from off-chip oscillator;
        reset: in std_logic; -- KEY0
        send: in std_logic; -- KEY1
        rx: in std_logic; -- GPIO_0[0]
        tx: out std_logic -- GPIO_0[1]
    );
end uart_test_impl1;


architecture arch of uart_test_impl1 is


  component LFSR 
    port (
      clk : in std_logic; 
      reset : in std_logic;
      z : out std_logic_vector (7 downto 0)
    );
  end component;


  component uart 
    generic(
      -- Default settings:
      -- 19,200 baud, 8 data bits, 1 stop bits, 2^5 FIFO
      DBIT: integer:=8;     -- # data bits
      SB_TICK: integer:=16; -- # ticks for stop bits, 16/24/32 for 1/1.5/2 stop bits
      DVSR: integer:= 163;  -- baud rate divisor; DVSR = 50M/(16*baud rate)
      -- Note: read page 174 in P.P. Chu's textbook; this is so, because
      -- my De2-115 board has an off chip oscillator that gives me 50 MHz;
      DVSR_BIT: integer:=9; -- # bits of DVSR
      FIFO_W: integer:=5    -- # addr bits of FIFO, # words in FIFO=2^FIFO_W
    );
    port(
      clk: in std_logic; -- 50 MHz in the case of DE2-115 board;
      reset: in std_logic;
      rd_uart, wr_uart: in std_logic;
      rx: in std_logic;
      w_data: in std_logic_vector(7 downto 0);
      tx_full, rx_empty: out std_logic;
      r_data: out std_logic_vector(7 downto 0);
      tx: out std_logic
    );
  end component;


  signal tx_full, rx_empty: std_logic;
  signal rec_data: std_logic_vector(7 downto 0);
  signal send_data: std_logic_vector(7 downto 0);  
   

begin


		-- instantiate LFSR;
    lfsr_inst: LFSR port map (
      clk => clk, 
      reset => reset,
      z => send_data
    );


    -- instantiate UART;
    uart_int: uart 
    generic map (
      DBIT => 8,     -- # data bits
      SB_TICK => 16, -- # ticks for stop bits, 16/24/32 for 1/1.5/2 stop bits
      DVSR => 1,     -- 163: baud rate divisor; DVSR = 50M/(16*baud rate)
      DVSR_BIT => 2, -- # bits of DVSR
      FIFO_W => 3    -- # 5: addr bits of FIFO, # words in FIFO=2^FIFO_W
    )
		port map (
      clk=>clk, 
      reset=>reset, 
      rd_uart=>send,
      wr_uart=>send, 
      rx=>rx,
      w_data=>send_data,
      tx_full=>tx_full, 
      rx_empty=>rx_empty,
      r_data=>rec_data, 
      tx=>tx
    );
    
end arch;
